#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Python -*-

import unittest

import RTC
import OpenRTM_aist
import DroneController
import drone_distance_cal

import sys


from mock import Mock
from mock import patch, MagicMock

class TestDroneController(unittest.TestCase):
    def setUp(self):
        self.m = DroneController.DroneController(0)

        self.xx_logger = Mock()
        DroneController.logger.warning = self.xx_logger.warning
        DroneController.logger.error = self.xx_logger.error
        DroneController.logger.info = self.xx_logger.info
        DroneController.logger.debug = self.xx_logger.debug

    def test_read_mag(self):
        xx = Mock()
        self.m._inMag.isNew = xx.isNew
        self.m._inMag.read = xx.read

        init_mag = RTC.TimedFloat(RTC.Time(0, 0), 0.0)
        mag = RTC.TimedFloat(RTC.Time(0, 0), 45.0)

        self.m._d_inMag = init_mag
        xx.read.return_value = mag

        in_is_new = [False,True]
        result = [0, 45.0]

        for i,d in enumerate(in_is_new):
            xx.isNew.return_value = in_is_new[i]
            self.m._read_mag()
            
            self.assertEqual(self.m._d_inMag.data, result[i])

    def test_set_target_speed(self):
        xx = Mock()

        init_speed = RTC.TimedFloat(RTC.Time(0, 0), 0.0)


        self.m._d_outTargetSpeed = init_speed


        speed = [5.0]
        result = [5.0]

        self.m._SpeedParam = speed[0]
        self.m._set_target_speed()
        self.assertEqual(self.m._d_outTargetSpeed.data, result[0])



    def test_set_target_yaw(self):
        xx = Mock()

        self.m._turn_dir = xx._turn_dir

        init_yaw = RTC.TimedShortSeq(RTC.Time(0, 0), [])

        self.m._d_outTargetYaw = init_yaw

        in_magcheck = [False, True]
        in_turndir = [-1, 0, 1]
        in_mag = [0.0, 1.0, 10.0, 90.0, 180.0, 270.0, 350.0, 359.0, 360.0]
        turnspeed = [45]
        self.m._TurnSpeed[0] = turnspeed[0]

        result = [
                  [[ [], [],  [], [], [], [], [], [], [] ],
                  [ [], [],  [], [], [], [], [], [], [] ],
                  [ [], [],  [], [], [], [], [], [], [] ]],

                  [[[-1, 315.0], [-1, 316.0],  [-1, 325.0], [-1,  45.0], [-1, 135.0], [-1, 225.0], [-1, 305.0], [-1, 314.0], [-1, 315.0]],
                  [[ 0,   0.0], [ 0,   1.0],  [ 0,  10.0], [ 0,  90.0], [ 0, 180.0], [ 0, 270.0], [ 0, 350.0], [ 0, 359.0], [ 0, 360.0]],
                  [[ 1,  45.0], [ 1,  46.0],  [ 1,  55.0], [ 1, 135.0], [ 1, 225.0], [ 1, 315.0], [ 1,  35.0], [ 1,  44.0], [ 1,  45.0]]]
                 ]

        for i1, d1 in enumerate(in_magcheck):
            self.m._mag = in_magcheck[i1]
            for i2, d2 in enumerate(in_turndir):
                xx._turn_dir.return_value = in_turndir[i2]
                for i3, d3 in enumerate(in_mag):
                    self.m._d_inMag.data = in_mag[i3]
                    self.m._set_target_yaw()
                    self.assertEqual(self.m._d_outTargetYaw.data, result[i1][i2][i3])

    def test_set_move_mode(self):
        xx = Mock()

        self.m._move_mode = xx._move_mode

        init_movemode = RTC.TimedString(RTC.Time(0, 0), "")

        self.m._d_outMoveMode = init_movemode

        in_move = [1, 2]
        result = ["Absolute", "Relative"]

        for i, d in enumerate(in_move):
            xx._move_mode.return_value = in_move[i]
            self.m._set_move_mode()
            self.assertEqual(self.m._d_outMoveMode.data, result[i])

    def test_move_speed(self):
        xx = Mock()

        self.m._check_drone_on_home = xx._check_drone_on_home

        in_home = [False, True] 
        in_button = [0,
                     DroneController.BUTTON_05_MASK,
                     DroneController.BUTTON_07_MASK,
                     DroneController.BUTTON_05_MASK | DroneController.BUTTON_07_MASK
                     ]

        in_speed = [1.0, 2.0, 5.0,  9.0, 10.0]
        result1 = [[[1.0, 2.0, 5.0,  9.0, 10.0],
                   [2.0, 3.0, 6.0, 10.0, 10.0],
                   [1.0, 1.0, 4.0,  8.0,  9.0],
                   [1.0, 2.0, 5.0,  9.0, 10.0]],

                   [[1.0, 2.0, 5.0,  9.0, 10.0],
                   [1.0, 2.0, 5.0,  9.0, 10.0],
                   [1.0, 2.0, 5.0,  9.0, 10.0],
                   [1.0, 2.0, 5.0,  9.0, 10.0]]
                   ]
        result2 = [[[0, 0, 0, 0, 0],
                   [0, 0, 0, 0, 0],
                   [0, 0, 0, 0, 0],
                   [1, 1, 1, 1, 1]],

                   [[0, 0, 0, 0, 0],
                   [1, 1, 1, 1, 1],
                   [1, 1, 1, 1, 1],
                   [1, 1, 1, 1, 1]]
                   ]

        for i1, d1 in enumerate(in_home):
            xx._check_drone_on_home.return_value = in_home[i1]        
            for i2, d2 in enumerate(in_button):
                self.m._d_ButtonIn.data = in_button[i2]
                for i3, d3 in enumerate(in_speed):
                    self.xx_logger.warning.call_count = 0
                    self.m._SpeedParam = in_speed[i3]

                    self.m._move_speed()

                    self.assertEqual(self.m._SpeedParam, result1[i1][i2][i3])
                    self.assertEqual(self.xx_logger.warning.call_count, result2[i1][i2][i3])

    def test_front_back(self):
        in_data = [[], [-2,0], [-1,0], [0,0], [1,0], [2,0]]
        result =  [ 0,      1,      1,     0,    -1,   -1]

        for i, d in enumerate(in_data):
            self.m._d_AnalogIn.data = in_data[i]

            self.assertEqual(self.m._front_back(), result[i])

    def test_move_continue(self):
        
        in_flont_back = [0, -1, 1, 0, 0, 0, 0, -1, 1, 0, -1, -1, 0, 1,
                        0, 0, 0, 0, 0, 0, -1, 0, 1, -1,
                        1, 0, -1, 1, 0, -1, 0, -1, 1, 0,
                        1, -1, 1, 0, -1, 0, -1, -1, 0, 0,
                        1, 0, 0, 0, 0, -1, 0, 0, 0, 0,
                        0, 0, 0
                        ]
        in_last_flont_back = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1, -1, -1,
                              0, 0, 0, 0, 0, 0, 1, 1, 1, 0,
                              0, 0, 0, 0, 0, -1, -1, -1, 1, 1,
                              1, 0, 0, 0, -1, 1, -1, -1, -1, -1,
                              1, 1, 1, 0, 0, 0, -1, 1, 0, -1,
                              -1, 0, 0
                              ]

        in_right_left = [0, 0, 0, -1, 1, 0, 0, 1, 0, -1, 1, 0, 0, 0,
                         -1, 0, 1, 0, 0, 0, -1, -1, 0, -1,
                         0, 1, 0, -1, 1, -1, 1, 1, -1, 1,
                         0, -1, 0, -1, -1, 1, -1, 0, -1, 0,
                         0, 0, -1, 1, 0, 0, 0, 0, 0, 0,
                         0, 0, 0
                         ]
        in_last_right_left = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                              1, 1, 1, 0, 0, 0, 0, 0, 0, -1,
                              -1, -1, 0, 0, 0, 1, 1, 1, 0, 0,
                              0, -1, -1, -1, -1, 1, -1, -1, -1, -1,
                              0, 0, 0, 1, 1, 1, -1, 0, 1, -1,
                              0, 1, 0
                              ]
        
        in_up_down = [0, 0, 0, 0, 0, -1, 1, 0, 1, 1, -1, 0, 0, 0,
                      0, 0, 0, -1, 0, 1, 0, 1, -1, 0,
                      -1, 1, -1, 0, 1, 0, 1, 0, 0, -1,
                      -1, 0, 1, 1, -1, 1, -1, 0, 0, 1,
                      0, 1, 0, 0, -1, 0, 0, 0, 0, 0,
                      0, 0, 0
                      ]
        in_last_up_down = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                           0, 0, 0, -1, -1, -1, 0, 0, 0, 0,
                           0, 0, 1, 1, 1, 0, 0, 0, -1, -1,
                           -1, 1, 1, 1, 0, 1, -1, 0, 0, 0,
                           1, 1, 1, -1, -1, -1, 0, 1, -1, -1,
                           0, 0, -1
                           ]



        result = [False, False, False, False, False, False, False, False, False, False, False, True, False, False,
                  False, False, True, True, False, False, False, False, False, False,
                  False, False, False, False, False, False, False, True, False, False,
                  True, False, False, True, False, False, True, False, False, False,
                  False, False, False, False, False, False, False, False, False, False,
                  False, False, False
                  ]


        for i1, d1 in enumerate(in_flont_back):
            self.m._last_front_back = in_last_flont_back[i1]
            self.m._last_right_left = in_last_right_left[i1]
            self.m._last_up_down = in_last_up_down[i1]
            moved_countinue = self.m._move_continue(in_flont_back[i1], in_right_left[i1], in_up_down[i1])
            self.assertEqual(moved_countinue, result[i1])

    def test_turn_dir(self):  
        xx = Mock()

        self.m._check_drone_on_home = xx._check_drone_on_home
        self.m._front_back = xx._front_back
        self.m._right_left = xx._right_left
        self.m._up_down = xx._up_down


        in_home = [False, True]
        in_data = [[0,-1], [0,0], [0,1]]
        in_move = [-1, 0, 1]

        result =  [[[[[0, 0, 0],[0, 0, 0],[0, 0, 0]], [[0, 0, 0],[0, -1, 0],[0, 0, 0]], [[0, 0, 0],[0, 0, 0],[0, 0, 0]]],
                   [[[ 0, 0, 0],[0, 0, 0],[0, 0, 0]], [[0, 0, 0],[ 0, 0, 0],[0, 0, 0]], [[0, 0, 0],[0, 0, 0],[0, 0, 0]]],
                   [[[ 0, 0, 0],[0, 0, 0],[0, 0, 0]], [[0, 0, 0],[0,  1, 0],[0, 0, 0]], [[0, 0, 0],[0, 0, 0],[0, 0, 0]]]],


                   [[[[0, 0, 0],[0, 0, 0],[0, 0, 0]], [[0, 0, 0],[0, 0, 0],[0, 0, 0]], [[0, 0, 0],[0, 0, 0],[0, 0, 0]]],
                   [[[ 0, 0, 0],[0, 0, 0],[0, 0, 0]], [[0, 0, 0],[0, 0, 0],[0, 0, 0]], [[0, 0, 0],[0, 0, 0],[0, 0, 0]]],
                   [[[ 0, 0, 0],[0, 0, 0],[0, 0, 0]], [[0, 0, 0],[0, 0, 0],[0, 0, 0]], [[0, 0, 0],[0, 0, 0],[0, 0, 0]]]]
                    ]

        for i1, d1 in enumerate(in_home):
            xx._check_drone_on_home.return_value = in_home[i1]
            for i2, d2 in enumerate(in_data):
                self.m._d_AnalogIn.data = in_data[i2]
                for i3, d3 in enumerate(in_move):
                    #front_back
                    xx._front_back.return_value =in_move[i3]
                    for i4, d4 in enumerate(in_move):
                        #right_left
                        xx._right_left.return_value = in_move[i4]
                        for i5, d5 in enumerate(in_move):
                            #up_down
                            xx._up_down.return_value = in_move[i5]

                            operation = self.m._turn_dir()

                            self.assertEqual(operation, result[i1][i2][i3][i4][i5])

    def test_move_mode(self):
        xx = Mock()

        self.m._check_drone_on_home = xx._check_drone_on_home
        self.m._front_back = xx._front_back
        self.m._right_left = xx._right_left
        self.m._up_down = xx._up_down

        in_home = [False, True]
        in_target_mode = [1, 2]
        in_move = [-1, 0, 1]
        in_button = [0, DroneController.BUTTON_10_MASK]

        result = [[[[[[1, 1], [1, 1], [1, 1]],  [[1, 1], [1, 1], [1, 1]], [[1, 1], [1, 1], [1, 1]]],
                  [[[1, 1], [1, 1], [1, 1]],  [[1, 1], [1, 2], [1, 1]], [[1, 1], [1, 1], [1, 1]]],
                  [[[1, 1], [1, 1], [1, 1]],  [[1, 1], [1, 1], [1, 1]], [[1, 1], [1, 1], [1, 1]]]],

                  [[[[2, 2], [2, 2], [2, 2]],  [[2, 2], [2, 2], [2, 2]], [[2, 2], [2, 2], [2, 2]]],
                  [[[2, 2], [2, 2], [2, 2]],  [[2, 2], [2, 1], [2, 2]], [[2, 2], [2, 2], [2, 2]]],
                  [[[2, 2], [2, 2], [2, 2]],  [[2, 2], [2, 2], [2, 2]], [[2, 2], [2, 2], [2, 2]]]]],



                  [[[[[1, 1], [1, 1], [1, 1]],  [[1, 1], [1, 1], [1, 1]], [[1, 1], [1, 1], [1, 1]]],
                  [[[1, 1], [1, 1], [1, 1]],  [[1, 1], [1, 1], [1, 1]], [[1, 1], [1, 1], [1, 1]]],
                  [[[1, 1], [1, 1], [1, 1]],  [[1, 1], [1, 1], [1, 1]], [[1, 1], [1, 1], [1, 1]]]],

                  [[[[2, 2], [2, 2], [2, 2]],  [[2, 2], [2, 2], [2, 2]], [[2, 2], [2, 2], [2, 2]]],
                  [[[2, 2], [2, 2], [2, 2]],  [[2, 2], [2, 2], [2, 2]], [[2, 2], [2, 2], [2, 2]]],
                  [[[2, 2], [2, 2], [2, 2]],  [[2, 2], [2, 2], [2, 2]], [[2, 2], [2, 2], [2, 2]]]]]
                  ]

        for i1, d1 in enumerate(in_home):
            xx._check_drone_on_home.return_value = in_home[i1]
            for i2, d2 in enumerate(in_target_mode):
                self.m._target_gps._mode = in_target_mode[i2]
                for i3, d3 in enumerate(in_move):
                    #front_back
                    xx._front_back.return_value =in_move[i3]
                    for i4, d4 in enumerate(in_move):
                        #right_left
                        xx._right_left.return_value = in_move[i4]
                        for i5, d5 in enumerate(in_move):
                            #up_down
                            xx._up_down.return_value = in_move[i5]
                            for i6, d6 in enumerate(in_button):
                                self.m._d_ButtonIn.data = in_button[i6]
                
                                mode = self.m._move_mode()
                
                                self.assertEqual(mode, result[i1][i2][i3][i4][i5][i6])

    def test_front_back_check(self):
        in_status = [-2, -1, 0, 1, 2]
        result = [-1, -1, 0, 1, 1]
        
        for i, d in enumerate(in_status):
            status = self.m._front_back_check(in_status[i])
            self.assertEqual(status, result[i])

    def test_turn_dir_check(self):
        in_dir = [-2, -1, 0, 1, 2]
        result = [-1, -1, 0, 1, 1]
        
        for i, d in enumerate(in_dir):
            dir = self.m._turn_dir_check(in_dir[i])
            self.assertEqual(dir, result[i])
            
    def test_move_speed_check(self):
        in_speed = [0.0, 1.0, 2.0, 5.0, 9.0, 10.0, 11.0]
        result  =  [1.0, 1.0, 2.0, 5.0, 9.0, 10.0, 10.0]
        
        for i, d in enumerate(in_speed):
            speed = self.m._move_speed_check(in_speed[i])
            self.assertEqual(speed, result[i])
            
                        
if __name__ == '__main__':
    unittest.main()